@@ -383,6 +383,12 @@ impl From<String> for Value {
     }
 }
 
+impl From<&str> for Value {
+    fn from(v: &str) -> Self {
+        v.to_owned().into()
+    }
+}
+
 impl From<Bin> for Value {
     fn from(v: Bin) -> Self {
         Self::Bin(v)
@@ -401,6 +407,12 @@ impl From<Vec<(Value, Value)>> for Value {
     }
 }
 
+impl From<Nil> for Value {
+    fn from(_: Nil) -> Self {
+        Self::Nil
+    }
+}
+
 #[doc(hidden)]
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Error)]
 #[error("value is not of the expected type")]
@@ -683,10 +695,14 @@ impl crate::Serializer for Serializer {
         Ok(())
     }
     fn serialize_map(&mut self, len: u32) -> Result<(), Self::Error> {
-        let vec = iter::repeat((Value::Nil, Value::Nil))
-            .take(len as usize)
-            .collect();
-        self.stack.push((0, Value::Map(vec)));
+        if len == 0 {
+            self.push_atom(Value::Map(vec![]));
+        } else {
+            let vec = iter::repeat((Value::Nil, Value::Nil))
+                .take(len as usize)
+                .collect();
+            self.stack.push((0, Value::Map(vec)));
+        }
         Ok(())
     }
     fn serialize_ext(&mut self, tag: i8, v: &[u8]) -> Result<(), Self::Error> {
@@ -890,3 +906,227 @@ impl crate::Deserialize for Any {
         Ok(Any)
     }
 }
+
+#[doc(hidden)]
+#[macro_export]
+macro_rules! msgpack_array {
+    ($acc: ident) => {
+    };
+    ($acc: ident,) => {
+    };
+    ($acc: ident nil) => {
+        $acc.push($crate::msgpack_value!(nil));
+    };
+    ($acc: ident nil, $( $rest: tt )*) => {
+        $acc.push($crate::msgpack_value!(nil));
+        $crate::msgpack_array!($acc $( $rest )*);
+    };
+    ($acc: ident [ $( $tt: tt )* ]) => {
+        $acc.push($crate::msgpack_value!([ $( $tt )* ]));
+    };
+    ($acc: ident [ $( $tt: tt )* ], $( $rest: tt )*) => {
+        $acc.push($crate::msgpack_value!([ $( $tt )* ]));
+        $crate::msgpack_array!($acc $( $rest )*);
+    };
+    ($acc: ident { $( $tt: tt )* }) => {
+        $acc.push($crate::msgpack_value!({ $( $tt )* }));
+    };
+    ($acc: ident { $( $tt: tt )* }, $( $rest: tt )*) => {
+        $acc.push($crate::msgpack_value!({ $( $tt )* }));
+        $crate::msgpack_array!($acc $( $rest )*);
+    };
+    ($acc: ident $expr: expr) => {
+        $acc.push($crate::msgpack_value!($expr));
+    };
+    ($acc: ident $expr: expr, $( $rest: tt )*) => {
+        $acc.push($crate::msgpack_value!($expr));
+        $crate::msgpack_array!($acc $( $rest )*);
+    };
+}
+
+#[doc(hidden)]
+#[macro_export]
+macro_rules! msgpack_map {
+    (@key $acc: ident []) => {
+    };
+    (@key $acc: ident [],) => {
+    };
+    (@key $acc: ident [ $( $key: tt )* ] : $( $rest: tt )*) => {
+        let key = $crate::msgpack_value!($( $key )*);
+        $crate::msgpack_map!(@val $acc key $( $rest )*);
+    };
+    (@key $acc: ident [ $( $key: tt )* ] $tt: tt $( $rest: tt )*) => {
+        $crate::msgpack_map!(@key $acc [ $( $key )* $tt ] $( $rest )*);
+    };
+    (@val $acc: ident $key: ident nil) => {
+        $acc.push(($key, $crate::msgpack_value!(nil)));
+    };
+    (@val $acc: ident $key: ident nil, $( $rest: tt )*) => {
+        $acc.push(($key, $crate::msgpack_value!(nil)));
+        $crate::msgpack_map!(@key $acc [] $( $rest )*);
+    };
+    (@val $acc: ident $key: ident [ $( $tt: tt )* ]) => {
+        $acc.push(($key, $crate::msgpack_value!([ $( $tt )* ])));
+    };
+    (@val $acc: ident $key: ident [ $( $tt: tt )* ], $( $rest: tt )*) => {
+        $acc.push(($key, $crate::msgpack_value!([ $( $tt )* ])));
+        $crate::msgpack_map!(@key $acc [] $( $rest )*);
+    };
+    (@val $acc: ident $key: ident { $( $tt: tt )* }) => {
+        $acc.push(($key, $crate::msgpack_value!({ $( $tt )* })));
+    };
+    (@val $acc: ident $key: ident { $( $tt: tt )* }, $( $rest: tt )*) => {
+        $acc.push(($key, $crate::msgpack_value!({ $( $tt )* })));
+        $crate::msgpack_map!(@key $acc [] $( $rest )*);
+    };
+    (@val $acc: ident $key: ident $expr: expr) => {
+        $acc.push(($key, $crate::msgpack_value!($expr)));
+    };
+    (@val $acc: ident $key: ident $expr: expr, $( $rest: tt )*) => {
+        $acc.push(($key, $crate::msgpack_value!($expr)));
+        $crate::msgpack_map!(@key $acc [] $( $rest )*);
+    };
+}
+
+#[doc(hidden)]
+#[macro_export]
+macro_rules! msgpack_value {
+    (nil) => {
+        $crate::value::Value::Nil
+    };
+    ([ $( $tt: tt )* ]) => {
+        {
+            #[allow(unused_mut)]
+            let mut array = vec![];
+            $crate::msgpack_array!(array $( $tt )*);
+            $crate::value::Value::Array(array)
+        }
+    };
+    ({ $( $tt: tt )* }) => {
+        {
+            #[allow(unused_mut)]
+            let mut map = vec![];
+            $crate::msgpack_map!(@key map [] $( $tt )*);
+            $crate::value::Value::Map(map)
+        }
+    };
+    ($other: expr) => {
+        $crate::value::Value::from($other)
+    };
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn msgpack_macro() {
+        assert_eq!(Value::Int(Int::from(42)), msgpack_value!(42));
+        assert_eq!(Value::Int(Int::from(-42)), msgpack_value!(-42));
+        assert_eq!(Value::F64(3.14), msgpack_value!(3.14));
+        assert_eq!(Value::F32(3.14), msgpack_value!(3.14f32));
+        assert_eq!(
+            Value::Str(Str("hello world".to_owned().into_bytes())),
+            msgpack_value!("hello world")
+        );
+        assert_eq!(Value::Bool(true), msgpack_value!(true));
+        assert_eq!(Value::Bool(false), msgpack_value!(false));
+        assert_eq!(Value::Nil, msgpack_value!(Nil));
+        assert_eq!(Value::Int(Int::from(7)), msgpack_value!(3 + 4));
+
+        assert_eq!(Value::Nil, msgpack_value!(nil));
+
+        assert_eq!(
+            Value::Array(vec![
+                msgpack_value!(42),
+                msgpack_value!(true),
+                msgpack_value!(nil),
+                msgpack_value!("hello"),
+            ]),
+            msgpack_value!([42, true, nil, "hello"])
+        );
+
+        assert_eq!(
+            Value::Array(vec![
+                msgpack_value!(42),
+                msgpack_value!(true),
+                msgpack_value!(nil),
+                msgpack_value!("hello"),
+            ]),
+            msgpack_value!([42, true, nil, "hello",])
+        );
+
+        assert_eq!(
+            Value::Array(vec![
+                msgpack_value!(42),
+                Value::Array(vec![
+                    Value::Array(vec![msgpack_value!(true)]),
+                    msgpack_value!(nil),
+                ]),
+                msgpack_value!("hello"),
+            ]),
+            msgpack_value!([42, [[true], nil], "hello",])
+        );
+
+        assert_eq!(Value::Array(vec![]), msgpack_value!([]));
+
+        assert_eq!(
+            Value::Map(vec![
+                (msgpack_value!(42), msgpack_value!(true)),
+                (msgpack_value!(nil), msgpack_value!("hello")),
+            ]),
+            msgpack_value!({ 42: true, nil: "hello", })
+        );
+
+        assert_eq!(
+            Value::Map(vec![
+                (msgpack_value!(0), msgpack_value!(nil)),
+                (msgpack_value!(1), msgpack_value!(nil)),
+            ]),
+            msgpack_value!({ 0: nil, 1: nil })
+        );
+
+        assert_eq!(
+            Value::Map(vec![
+                (msgpack_value!(0), msgpack_value!({})),
+                (msgpack_value!(1), msgpack_value!({})),
+            ]),
+            msgpack_value!({ 0: {}, 1: {} })
+        );
+
+        assert_eq!(
+            Value::Map(vec![
+                (msgpack_value!(0), msgpack_value!([])),
+                (msgpack_value!(1), msgpack_value!([])),
+            ]),
+            msgpack_value!({ 0: [], 1: [] })
+        );
+
+        assert_eq!(
+            Value::Map(vec![
+                (msgpack_value!(0), msgpack_value!(-1)),
+                (msgpack_value!(-1), msgpack_value!(0)),
+            ]),
+            msgpack_value!({ 0: -1, -1: 0 })
+        );
+
+        assert_eq!(
+            Value::Map(vec![
+                (msgpack_value!(42), msgpack_value!({ true: false })),
+                (msgpack_value!({ nil: 3.14 }), msgpack_value!("hello")),
+            ]),
+            msgpack_value!({ 42: { true: false }, { nil: 3.14 }: "hello", })
+        );
+        assert_eq!(Value::Map(vec![]), msgpack_value!({}));
+
+        assert_eq!(
+            Value::Bin(Bin(vec![0xDEu8, 0xAD, 0xBE, 0xEF])),
+            msgpack_value!(Bin(vec![0xDE, 0xAD, 0xBE, 0xEF]))
+        );
+
+        assert_eq!(
+            Value::Array(vec![msgpack_value!(-42)]),
+            msgpack_value!([-42])
+        );
+    }
+}
